# Модули
Модуль - функционально законченный фрагмент программы. Что это значит для нас? 
В этой библиотеке модуль тоже является компонентом, но он состоит из набора других компонентов. В идеале модуль содержит в себе компоненты для типов, и в некоторых случаях модуля.
Существует третий вид компонентов - сборки, но о них описано в следующей главе.

Для большей понятности:
Можно создать модуль регистрации, в который входят такие типы, как классы для показа окон регистрации, классы для работы с сервером регистрации, вспомогательные классы.

## Объявление
прежде чем использовать модуль, его нужно объявить. Так как модуль является следующей ступенью над типом, а тип в библиотеке представляется в виде набора функций, логично представить модуль в виде класса. Чтобы объявить модуль нужно:
* Создать свой класс
* Объявить, что класс реализует протокол `DIModule`
* Реализовать метод `load(builder: DIContainerBuilder)`

```Swift
class RegistrationModule: DIModule {
  func load(builder: DIContainerBuilder) {
    builder.register...
  }
}
```
Слово builder вам уже знакомо из прошлых глав.

## Регистрация
После того как мы объявили модуль, его нужно зарегистрировать для дальнейшего использования:
```Swift
builder.register(module: RegistrationModule())
```
Регистрация модуля, не сильно отличается от регистрации типа, но в функцию регистрации добавляется ключевое слово `module`. Помимо этого стоит обратить внимание на то, что регистрирует не класс, а экземпляр класса. На самом деле сколько бы раз не был бы создан экземпляр модуля, при регистрации он будет учитываться только один раз.


#### [Главная](main.md)
#### [Предыдущая глава "Время жизни"](lifetime.md)
#### [Следующая глава "Сборки"](assembly.md)